@@ -33,6 +33,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/list.h>
 #include <gnuastro/data.h>
 #include <gnuastro/table.h>
+#include <gnuastro/blank.h>
+#include <gnuastro/threads.h>
 #include <gnuastro/arithmetic.h>
 
 #include <gnuastro-internal/timing.h>
@@ -155,114 +157,6 @@ options_get_home()
 
 
 
-/* The input to this function is a string of any number of numbers
-   separated by a comma (`,') and possibly containing fractions, for
-   example: `1,2/3, 4.95'. The output `gal_data_t' contains the array of
-   given values in `double' type. You can read the number from its `size'
-   element. */
-gal_data_t *
-gal_options_parse_list_of_numbers(char *string, char *filename, size_t lineno)
-{
-  size_t i, num=0;
-  gal_data_t *out;
-  char *c=string, *tailptr;
-  gal_list_f64_t *list=NULL, *tdll;
-  double numerator=NAN, denominator=NAN, tmp;
-
-
-  /* The nature of the arrays/numbers read here is very small, so since
-     `p->cp.minmapsize' might not have been read yet, we will set it to -1
-     (largest size_t number), so the values are kept in memory. */
-  size_t minmapsize=-1;
-
-
-  /* Go through the input character by character. */
-  while(*c!='\0')
-    {
-      switch(*c)
-        {
-
-        /* Ignore space or tab. */
-        case ' ':
-        case '\t':
-          ++c;
-          break;
-
-        /* Comma marks the transition to the next number. */
-        case ',':
-          if(isnan(numerator))
-            error_at_line(EXIT_FAILURE, 0, filename, lineno, "a number "
-                          "must be given before `,'. You have given: `%s'",
-                          string);
-          gal_list_f64_add(&list, isnan(denominator)
-                           ? numerator : numerator/denominator);
-          numerator=denominator=NAN;
-          ++num;
-          ++c;
-          break;
-
-        /* Divide two numbers. */
-        case '/':
-          if( isnan(numerator) || !isnan(denominator) )
-            error_at_line(EXIT_FAILURE, 0, filename, lineno, "`/' must "
-                          "only be between two numbers and used for "
-                          "division. But you have given `%s'", string);
-          ++c;
-          break;
-
-        /* Read the number. */
-        default:
-
-          /* Parse the string. */
-          tmp=strtod(c, &tailptr);
-          if(tailptr==c)
-            error_at_line(EXIT_FAILURE, 0, filename, lineno, "the first "
-                          "part of `%s' couldn't be read as a number. This "
-                          "was part of `%s'", c, string);
-
-          /* See if the number should be put in the numerator or
-             denominator. */
-          if(isnan(numerator)) numerator=tmp;
-          else
-            {
-              if(isnan(denominator)) denominator=tmp;
-              else error_at_line(EXIT_FAILURE, 0, filename, lineno, "more "
-                                 "than two numbers in each element.");
-            }
-
-          /* Set `c' to tailptr. */
-          c=tailptr;
-        }
-    }
-
-
-  /* If the last number wasn't finished by a `,', add the read value to the
-     list */
-  if( !isnan(numerator) )
-    {
-      ++num;
-      gal_list_f64_add(&list, isnan(denominator)
-                       ? numerator : numerator/denominator);
-    }
-
-
-  /* Allocate the output data structure and fill it up. */
-  i=num;
-  out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &num, NULL, 0,
-                     minmapsize, NULL, NULL, NULL);
-  for(tdll=list;tdll!=NULL;tdll=tdll->next)
-    ((double *)(out->array))[--i]=tdll->v;
-
-
-  /* Clean up and return. */
-  gal_list_f64_free(list);
-  return out;
-}
-
-
-
-
-
 
 
 
@@ -327,6 +221,7 @@ gal_options_print_citation(struct argp_option *option, char *arg,
                            char *filename, size_t lineno, void *pa)
 {
   struct gal_options_common_params *cp=(struct gal_options_common_params *)pa;
+  char *gnuastro_acknowledgement;
   char *gnuastro_bibtex=
     "Gnuastro package/infrastructure\n"
     "-------------------------------\n"
@@ -355,17 +250,35 @@ gal_options_print_citation(struct argp_option *option, char *arg,
   /* Print the statements. */
   printf("\nThank you for using %s (%s) %s.\n\n", cp->program_name,
          PACKAGE_NAME, PACKAGE_VERSION);
-  printf("Citations are vital for the continued work on Gnuastro.\n\n"
-         "Please cite these BibTeX record(s) in your paper(s).\n"
-         "(don't forget to also include the version as shown above)\n\n"
-         "%s\n\n",
-         gnuastro_bibtex);
+  printf("Citations and acknowledgement are vital for the continued "
+         "work on Gnuastro.\n\n"
+         "Please cite the following record(s) and add the acknowledgement "
+         "statement below in your work to support us. Please note that "
+         "different Gnuastro programs may have different corresponding "
+         "papers. Hence, please check all the programs you used. Don't "
+         "forget to also include the version as shown above for "
+         "reproducibility.\n\n"
+         "%s\n\n", gnuastro_bibtex);
 
 
   /* Only print the citation for the program if one exists. */
   if(cp->program_bibtex[0]!='\0') printf("%s\n\n", cp->program_bibtex);
 
 
+  /* Notice for acknowledgements. */
+  asprintf(&gnuastro_acknowledgement,
+           "Acknowledgement\n"
+           "---------------\n"
+           "This work was partly done using GNU Astronomy Utilities "
+           "(Gnuastro) version %s. Gnuastro is a generic package for "
+           "astronomical data manipulation and analysis which was "
+           "primarily created and developed for research funded by the "
+           "Monbukagakusho (Japanese government) scholarship and ERC "
+           "advanced grant 339659-MUSICOS.", PACKAGE_VERSION);
+  printf("%s\n", gnuastro_acknowledgement);
+  free(gnuastro_acknowledgement);
+
+
   /* Print a thank you message. */
   printf("                                               ,\n"
          "                                              {|'--.\n"
@@ -509,12 +422,304 @@ gal_options_read_tableformat(struct argp_option *option, char *arg,
 
 
 
+/* The input to this function is a string of any number of numbers
+   separated by a comma (`,') and possibly containing fractions, for
+   example: `1,2/3, 4.95'. The output `gal_data_t' contains the array of
+   given values in `double' type. You can read the number from its `size'
+   element. */
+gal_data_t *
+gal_options_parse_list_of_numbers(char *string, char *filename, size_t lineno)
+{
+  size_t i, num=0;
+  gal_data_t *out;
+  char *c=string, *tailptr;
+  gal_list_f64_t *list=NULL, *tdll;
+  double numerator=NAN, denominator=NAN, tmp;
+
+
+  /* The nature of the arrays/numbers read here is very small, so since
+     `p->cp.minmapsize' might not have been read yet, we will set it to -1
+     (largest size_t number), so the values are kept in memory. */
+  size_t minmapsize=-1;
+
+  /* Go through the input character by character. */
+  while(string && *c!='\0')
+    {
+      switch(*c)
+        {
+
+        /* Ignore space or tab. */
+        case ' ':
+        case '\t':
+          ++c;
+          break;
+
+        /* Comma marks the transition to the next number. */
+        case ',':
+          if(isnan(numerator))
+            error_at_line(EXIT_FAILURE, 0, filename, lineno, "a number "
+                          "must be given before `,'. You have given: `%s'",
+                          string);
+          gal_list_f64_add(&list, isnan(denominator)
+                           ? numerator : numerator/denominator);
+          numerator=denominator=NAN;
+          ++num;
+          ++c;
+          break;
+
+        /* Divide two numbers. */
+        case '/':
+          if( isnan(numerator) || !isnan(denominator) )
+            error_at_line(EXIT_FAILURE, 0, filename, lineno, "`/' must "
+                          "only be between two numbers and used for "
+                          "division. But you have given `%s'", string);
+          ++c;
+          break;
+
+        /* Extra dot is an error (cases like 2.5.5). Valid `.'s will be
+           read by `strtod'. */
+        case '.':
+          error_at_line(EXIT_FAILURE, 0, filename, lineno, "extra `.' in "
+                        "`%s'", string);
+          break;
+
+        /* Read the number. */
+        default:
+
+          /* Parse the string. */
+          tmp=strtod(c, &tailptr);
+          if(tailptr==c)
+            error_at_line(EXIT_FAILURE, 0, filename, lineno, "the first "
+                          "part of `%s' couldn't be read as a number. This "
+                          "was part of `%s'", c, string);
+
+          /* See if the number should be put in the numerator or
+             denominator. */
+          if(isnan(numerator)) numerator=tmp;
+          else
+            {
+              if(isnan(denominator)) denominator=tmp;
+              else error_at_line(EXIT_FAILURE, 0, filename, lineno, "more "
+                                 "than two numbers in each element.");
+            }
+
+          /* Set `c' to tailptr. */
+          c=tailptr;
+        }
+    }
+
+
+  /* If the last number wasn't finished by a `,', add the read value to the
+     list */
+  if( !isnan(numerator) )
+    {
+      ++num;
+      gal_list_f64_add(&list, isnan(denominator)
+                       ? numerator : numerator/denominator);
+    }
+
+
+  /* Allocate the output data structure and fill it up. */
+  if(num)
+    {
+      i=num;
+      out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &num, NULL, 0,
+                         minmapsize, NULL, NULL, NULL);
+      for(tdll=list;tdll!=NULL;tdll=tdll->next)
+        ((double *)(out->array))[--i]=tdll->v;
+    }
+  else
+    {
+      /* It is not possible to allocate a dataset with a size of 0 along
+         any dimension (in C it's possible, but conceptually it isn't). So,
+         we'll allocate space for one element, then free it. */
+      i=1;
+      out=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &i, NULL, 0,
+                         minmapsize, NULL, NULL, NULL);
+      out->size=out->dsize[0]=0;
+      free(out->array);
+      out->array=NULL;
+    }
+
+
+  /* Clean up and return. */
+  gal_list_f64_free(list);
+  return out;
+}
+
+
+
+
+
+/* The input to this function is a string of any number of strings
+   separated by a comma (`,') for example: `a,abc,abcd'. The output
+   `gal_data_t' contains the array of given strings. You can read the
+   number of inputs from its `size' element. */
+gal_data_t *
+gal_options_parse_csv_strings_raw(char *string, char *filename, size_t lineno)
+{
+  size_t i, num;
+  gal_data_t *out;
+  char *c=string, *str=NULL;
+  gal_list_str_t *list=NULL, *tstrll=NULL;
+
+
+  /* The nature of the arrays/numbers read here is very small, so since
+     `p->cp.minmapsize' might not have been read yet, we will set it to -1
+     (largest size_t number), so the values are kept in memory. */
+  size_t minmapsize=-1;
+
+
+  /* Go through the input character by character. */
+  while(string && *c!='\0')
+    {
+      switch(*c)
+        {
+        /* Comma marks the transition to the next string. */
+        case ',':
+          if(str==NULL)
+            error_at_line(EXIT_FAILURE, 0, filename, lineno, "a string "
+                          "must exist before the first `,'. You have "
+                          "given: `%s'", string);
+          *c='\0';
+          gal_list_str_add(&list, str, 1);
+          str=NULL;  /* Mark that the next character is the start */
+          break;
+
+        /* If the character isn't a coma, it is either in the middle of a
+           string at the start of it. If `str==NULL', then it is at the
+           start. */
+        default: if(str==NULL) str=c;
+        }
+
+      /* Increment C. */
+      ++c;
+    }
+
+
+  /* If the last element wasn't a comma, the last string hasn't been added
+     to the list yet. */
+  if(str) gal_list_str_add(&list, str, 1);
+
+
+  /* Allocate the output data structure and fill it up. */
+  if(list)
+    {
+      i=num=gal_list_str_number(list);
+      out=gal_data_alloc(NULL, GAL_TYPE_STRING, 1, &num, NULL, 0,
+                         minmapsize, NULL, NULL, NULL);
+      for(tstrll=list;tstrll!=NULL;tstrll=tstrll->next)
+        ((char **)(out->array))[--i]=tstrll->v;
+    }
+  else
+    {
+      /* It is not possible to allocate a dataset with a size of 0 along
+         any dimension (in C it's possible, but conceptually it isn't). So,
+         we'll allocate space for one element, then free it. */
+      i=1;
+      out=gal_data_alloc(NULL, GAL_TYPE_STRING, 1, &i, NULL, 0,
+                         minmapsize, NULL, NULL, NULL);
+      out->size=out->dsize[0]=0;
+      free(out->array);
+      out->array=NULL;
+    }
+
+
+  /* Clean up and return. Note that we don't want to free the space of
+     each string becuse it has been passed  */
+  gal_list_str_free(list, 0);
+  return out;
+}
+
+
+
+
+
+/* `arg' is the value given to an option. It contains multiple strings
+   separated by a comma (`,'). This function will parse `arg' and make a
+   `gal_data_t' array of strings from it. The output `gal_data_t' will be
+   put in `option->value'. */
+void *
+gal_options_parse_csv_strings(struct argp_option *option, char *arg,
+                              char *filename, size_t lineno, void *junk)
+{
+  size_t nc;
+  char *c, **strarr;
+  int i, has_space=0;
+  gal_data_t *values;
+  char *str, sstr[GAL_OPTIONS_STATIC_MEM_FOR_VALUES];
+
+  /* We want to print the stored values. */
+  if(lineno==-1)
+    {
+      /* Set the pointer to the values dataset. */
+      values = *(gal_data_t **)(option->value);
+
+      /* See if there are any space characters in the final string. */
+      strarr=values->array;
+      for(i=0;i<values->size;++i)
+        if(has_space==0)
+        {
+          for(c=strarr[i];*c!='\0';++c)
+            if(*c==' ' || *c=='\t')
+              {
+                has_space=1;
+                break;
+              }
+        }
+
+      /* If there is a space, the string must start wth quotation marks. */
+      nc = has_space ? 1 : 0;
+      if(has_space) {sstr[0]='"'; sstr[1]='\0';}
+
+
+      /* Write each string into the output string */
+      for(i=0;i<values->size;++i)
+        {
+          if( nc > GAL_OPTIONS_STATIC_MEM_FOR_VALUES-100 )
+            error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s so we "
+                  "can address the problem. The number of necessary "
+                  "characters in the statically allocated string has become "
+                  "too close to %d", __func__, PACKAGE_BUGREPORT,
+                  GAL_OPTIONS_STATIC_MEM_FOR_VALUES);
+          nc += sprintf(sstr+nc, "%s,", strarr[i]);
+        }
+
+      /* If there was a space, we need a quotation mark at the end of the
+         string. */
+      if(has_space) { sstr[nc-1]='"'; sstr[nc]='\0'; }
+      else            sstr[nc-1]='\0';
+
+      /* Copy the string into a dynamically allocated space, because it
+         will be freed later.*/
+      gal_checkset_allocate_copy(sstr, &str);
+      return str;
+    }
+
+  /* We want to read the user's string. */
+  else
+    {
+      /* If the option is already set, just return. */
+      if(option->set) return NULL;
+
+      /* Read the values. */
+      values=gal_options_parse_csv_strings_raw(arg, filename, lineno);
+
+      /* Put the values into the option. */
+      *(gal_data_t **)(option->value) = values;
+      return NULL;
+    }
+}
+
+
+
+
+
 /* Parse the given string into a series of size values (integers, stored as
-   an array of size_t). The ouput array will be stored in the `value'
-   element of the option. The last element of the array is `-1' to allow
-   finding the number of elements within it later (similar to a string
-   which terminates with a '\0' element). */
-#define PARSE_SIZES_STATICSTR_LEN 2000
+   an array of size_t). The output array will be stored in the `value'
+   element of the option. The last element of the array is
+   `GAL_BLANK_SIZE_T' to allow finding the number of elements within it
+   later (similar to a string which terminates with a '\0' element). */
 void *
 gal_options_parse_sizes_reverse(struct argp_option *option, char *arg,
                                 char *filename, size_t lineno, void *junk)
@@ -523,7 +728,7 @@ gal_options_parse_sizes_reverse(struct argp_option *option, char *arg,
   double *v;
   gal_data_t *values;
   size_t nc, num, *array;
-  char *str, sstr[PARSE_SIZES_STATICSTR_LEN];
+  char *str, sstr[GAL_OPTIONS_STATIC_MEM_FOR_VALUES];
 
   /* We want to print the stored values. */
   if(lineno==-1)
@@ -537,12 +742,12 @@ gal_options_parse_sizes_reverse(struct argp_option *option, char *arg,
       nc=0;
       for(i=num-1;i>=0;--i)
         {
-          if( nc > PARSE_SIZES_STATICSTR_LEN-100 )
+          if( nc > GAL_OPTIONS_STATIC_MEM_FOR_VALUES-100 )
             error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s so we "
                   "can address the problem. The number of necessary "
                   "characters in the statically allocated string has become "
                   "too close to %d", __func__, PACKAGE_BUGREPORT,
-                  PARSE_SIZES_STATICSTR_LEN);
+                  GAL_OPTIONS_STATIC_MEM_FOR_VALUES);
           nc += sprintf(sstr+nc, "%zu,", array[i]);
         }
       sstr[nc-1]='\0';
@@ -567,14 +772,17 @@ gal_options_parse_sizes_reverse(struct argp_option *option, char *arg,
       for(i=0;i<values->size;++i)
         {
           if(v[i]<0)
-            error(EXIT_FAILURE, 0, "the given value in `%s' (%g) is not 0 "
-                  "or positive. The values to the `--%s' option must be "
-                  "positive", arg, v[i], option->name);
+            error_at_line(EXIT_FAILURE, 0, filename, lineno, "a given "
+                          "value in `%s' (%g) is not 0 or positive. The "
+                          "values to the `--%s' option must be positive",
+                          arg, v[i], option->name);
+
 
           if(ceil(v[i]) != v[i])
-            error(EXIT_FAILURE, 0, "the given value in `%s' (%g) is not an "
-                  "integer. The values to the `--%s' option must be "
-                  "integers", arg, v[i], option->name);
+            error_at_line(EXIT_FAILURE, 0, filename, lineno, "a given "
+                          "value in `%s' (%g) is not an integer. The "
+                          "values to the `--%s' option must be integers",
+                          arg, v[i], option->name);
         }
 
       /* Write the values into an allocated size_t array and finish it with
@@ -582,7 +790,7 @@ gal_options_parse_sizes_reverse(struct argp_option *option, char *arg,
       num=values->size;
       array=gal_data_malloc_array(GAL_TYPE_SIZE_T, num+1, __func__, "array");
       for(i=0;i<num;++i) array[num-1-i]=v[i];
-      array[num] = (size_t)(-1);
+      array[num] = GAL_BLANK_SIZE_T;
 
       /* Put the array of size_t into the option, clean up and return.*/
       *(size_t **)(option->value) = array;
@@ -595,6 +803,65 @@ gal_options_parse_sizes_reverse(struct argp_option *option, char *arg,
 
 
 
+/* Parse options with values of a list of numbers. */
+void *
+gal_options_parse_csv_float64(struct argp_option *option, char *arg,
+                              char *filename, size_t lineno, void *junk)
+{
+  size_t i, nc;
+  double *darray;
+  gal_data_t *values;
+  char *str, sstr[GAL_OPTIONS_STATIC_MEM_FOR_VALUES];
+
+  /* We want to print the stored values. */
+  if(lineno==-1)
+    {
+      /* Set the pointer to the values dataset. */
+      values = *(gal_data_t **)(option->value);
+      darray=values->array;
+
+      /* Write each string into the output string */
+      nc=0;
+      for(i=0;i<values->size;++i)
+        {
+          if( nc > GAL_OPTIONS_STATIC_MEM_FOR_VALUES-100 )
+            error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s so we "
+                  "can address the problem. The number of necessary "
+                  "characters in the statically allocated string has become "
+                  "too close to %d", __func__, PACKAGE_BUGREPORT,
+                  GAL_OPTIONS_STATIC_MEM_FOR_VALUES);
+          nc += sprintf(sstr+nc, "%g,", darray[i]);
+        }
+      sstr[nc-1]='\0';
+
+      /* Copy the string into a dynamically allocated space, because it
+         will be freed later.*/
+      gal_checkset_allocate_copy(sstr, &str);
+      return str;
+    }
+
+  /* We want to read the user's string. */
+  else
+    {
+      /* If the option is already set, just return. */
+      if(option->set) return NULL;
+
+      /* Read the values. */
+      values=gal_options_parse_list_of_numbers(arg, filename, lineno);
+
+      /* Put the values into the option. */
+      *(gal_data_t **)(option->value) = values;
+
+      /* The return value is only for printing mode, so we can return
+         NULL after reading is complete. */
+      return NULL;
+    }
+}
+
+
+
+
+
 /* Two numbers must be provided as an argument. This function will read
    them as the sigma-clipping multiple and parameter and store the two in a
    2-element array. `option->value' must point to an already allocated
@@ -1433,6 +1700,11 @@ gal_options_read_config_set(struct gal_options_common_params *cp)
 
   /* Abort if any of the mandatory options are not set. */
   gal_options_abort_if_mandatory_missing(cp);
+
+  /* If `numthreads' is 0, use the number of threads available to the
+     system. */
+  if(cp->numthreads==0)
+    cp->numthreads=gal_threads_number();
 }
 
 
@@ -1722,7 +1994,7 @@ options_print_all(struct gal_options_common_params *cp, char *dirname,
       asprintf(&filename, "%s/%s.conf", dirname, cp->program_exec);
 
       /* Remove the file if it already exists. */
-      gal_checkset_check_remove_file(filename, 0, 0);
+      gal_checkset_writable_remove(filename, 0, 0);
 
       /* Open the file for writing */
       errno=0;
